
//-------------------------------------------------------------------
// DIForm.pas      DiskInfo32 Main Window     5.9.2004  by Gerd Platl
//-------------------------------------------------------------------
{$I comp_opt.inc}

unit DIForm;

interface

uses Windows, Forms, Menus, StdCtrls, ComCtrls, Classes, Controls,
     Dialogs, ImgList, FileCtrl,
     SysUtil2;

type
  TDriveListForm = class(TForm)
    MainMenu: TMainMenu;
      WindowMenu: TMenuItem;
        PrintSetup: TMenuItem;
        PrintCommand: TMenuItem;
        Refresh: TMenuItem;
        N1: TMenuItem;
        FileExit: TMenuItem;
      ViewMenu: TMenuItem;
        ViewKByte: TMenuItem;
        ViewMByte: TMenuItem;
        ViewBytes: TMenuItem;
        ViewGigaByte: TMenuItem;
      HelpMenu: TMenuItem;
        AboutItem: TMenuItem;

    diskInfoListView: TListView;
    DriveListBox: TDriveComboBox;
    StatusBar: TStatusBar;
    PrinterSetupDialog: TPrinterSetupDialog;
    PrintDialog: TPrintDialog;
    DriveListIcons: TImageList;

    procedure FormCreate(Sender: TObject);
    procedure HelpAbout(Sender: TObject);
    procedure ShowHint(Sender: TObject);
    procedure RefreshClick(Sender: TObject);
    procedure PrintSetupClick(Sender: TObject);
    procedure PrintCommandClick(Sender: TObject);
    procedure FileExitClick(Sender: TObject);
    procedure UnitClick(Sender: TObject);
  private
    lines: word;
    divisor: int64;
  end;

var
  DriveListForm: TDriveListForm;

implementation

uses SysUtils, DIAbout;

{$R *.DFM}

{*********************************************************}
{ get drive parameter list and set width and height
{---------------------------------------------------------}
procedure TDriveListForm.FormCreate(Sender: TObject);
var   h,w,i: integer;
begin
  divisor := 1024 * 1024;

  RefreshClick (Sender);

  // calculate client width
  w := 0;
  for i := 0 to 6
  do w := w + diskInfoListView.Columns[i].width;
  ClientWidth := w + 6;

  h := (DriveListBox.Height + 6) * lines;
  if h > ClientHeight
  then ClientHeight := h;
end;
{*********************************************************}
procedure TDriveListForm.HelpAbout(Sender: TObject);
begin
  AboutBox.ShowModal;
end;
{*********************************************************}
procedure TDriveListForm.ShowHint(Sender: TObject);
begin
  StatusBar.SimpleText := Application.Hint;
end;
{*********************************************************}
procedure TDriveListForm.PrintCommandClick(Sender: TObject);
begin
  if PrintDialog.Execute then Print;
end;
{*********************************************************}
procedure TDriveListForm.PrintSetupClick(Sender: TObject);
begin
  PrinterSetupDialog.Execute;
end;
{*********************************************************}
{ get drive information and refresh drive item list
{---------------------------------------------------------}
procedure TDriveListForm.RefreshClick(Sender: TObject);
const driveTypeName: array [0..DRIVE_RAMDISK] of string[20]
      =('unknown',  'floppy disk', 'swap drive'
       ,'harddisk', 'network drive', 'CD-drive', 'RAM-Disk');

const driveImageIndex: array [0..DRIVE_RAMDISK] of integer
      =(0,0,1,2,3,4,5);
var
  i, drive, maxDrive: word;
  bSize, bFree, bUsed, percent: int64;
  driveChar: char;
  Laufwerk: PChar;
  driveType: integer;
  volNameSize, fileSystemNameSize : dword;
  volName: string[255];

begin
  if (StatusBar.Top - diskInfoListView.Top) < diskInfoListView.Height
  then diskInfoListView.Height := StatusBar.Top - diskInfoListView.Top;
  Screen.Cursor := crHourGlass;
  volName := '?';
  try
    Application.OnHint := ShowHint;
    maxDrive := DriveListBox.Items.Count;
    diskInfoListView.visible := false;
    diskInfoListView.items.Clear;
    lines := 0;
    For i := 0 to  maxDrive - 1
    do begin
      driveChar := Uppercase(Copy(DriveListBox.Items[i],0,1))[1];
      drive := ord(driveChar) - 64;
      Laufwerk := PChar(driveChar + ':\');
      percent := 0;
      if DriveReady (drive, bSize, bFree)
      then with diskInfoListView.Items.Add
      do begin
        inc (lines);
        bSize := bSize div divisor;
        bFree := bFree div divisor;
        bUsed := bSize - bFree;
        if bSize > 0
        then percent := bFree * 100 div bSize;

        if GetVolumeInformation (Laufwerk, @volName[1], SizeOf(volName), nil
                                ,volNameSize, fileSystemNameSize, nil, 0)
        then volName[0] := char(volNameSize)
        else volName := '';
        driveType := GetDriveType(Laufwerk);

        Caption := ' ' + driveChar + ':';
        if driveType > DRIVE_RAMDISK
        then driveType := 0;
        ImageIndex := driveImageIndex[driveType];
        Subitems.Add (driveTypeName[driveType]);
        SubItems.Add (volName);
        SubItems.Add (IntDotString(bSize));
        SubItems.Add (IntDotString(bUsed));
        SubItems.Add (IntDotString(bFree));
        SubItems.Add (IntToStr (percent));
      end;
    end;
  finally
    diskInfoListView.visible := true;
    Screen.Cursor := crDefault;
  end;
end;
{*********************************************************}
{ menu command: File / Exit   closes application
{---------------------------------------------------------}
procedure TDriveListForm.FileExitClick(Sender: TObject);
begin
  Close;
end;

{*********************************************************}
{ change data display unit
{---------------------------------------------------------}
procedure TDriveListForm.UnitClick(Sender: TObject);
const unitText: array[0..3] of string
      = ('Bytes', 'KiloBytes', 'MegaBytes', 'GigaBytes');
begin
  case TMenuItem(Sender).tag of
    0: divisor := 1;
    1: divisor := 1024;
    2: divisor := 1024 * 1024;
    3: divisor := 1024 * 1024 * 1024;
  end;
  RefreshClick (sender);
  Hint := 'all values are displayed in ' + unitText[TMenuItem(Sender).tag];
end;

end.

